import java.awt.*;
import javax.swing.*;
import java.awt.event.*;

class Turtle
{
    /**x-Koordinate der Turtle*/
    protected int posX;
    /**y-Koordinate der Turtle*/
    protected int posY;     
    /**Winkel der Turtle im Gradmass*/
    protected int winkel;            
    /**Startkoordinate der Turtle*/
    protected int homeX;
    /**Startkoordinate der Turtle*/
    protected int homeY;
    /**Startwinkel der Turtle*/
    protected int homeWinkel; 
    /**Farbe der Turtle*/
    protected Color farbe;
    /**Stiftposition*/
    protected boolean stiftUnten;
    /**Modus schnell/langsam*/
    protected boolean schnell;
    /**Taktdauer im langsam-Modus*/
    protected int wartezeit;

    private Turtlesymbol symbol;
    private int breiteX, hoeheY;
    private static Oberflaeche oberflaeche = null;
    private Graphics g;

    /**
     * Konstruktor der Turtle
     * Erzeugt eine Turtle und versetzt sie in einen gültigen Startzustand.
     */

    public Turtle()
    {
        if (oberflaeche==null)
        {
            oberflaeche = new Oberflaeche();
        }
        posX=100;
        posY=200;
        winkel=0;
        homeX=posX;
        homeY=posY;
        homeWinkel=winkel;
        farbe = Color.black;

        stiftUnten = true;
        breiteX = oberflaeche.getBounds().width;
        hoeheY = oberflaeche.getBounds().height;

        symbol=new Turtlesymbol(oberflaeche,posX,posY,winkel);
        symbol.StiftfarbeSetzen(Color.black);

        oberflaeche.LeinwandAktivieren();

        symbol.setVisible(false);
        symbol.PositionSetzen(posX, posY, winkel);

        symbol.setVisible(true);
        schnell = true;
        wartezeit = 2;
    }

    /**
     * Setzt die Turtle wieder an ihre Ausgangsposition.
     */
    public void ZumStartpunktGehen()
    {
        posX=homeX;
        posY=homeY;
        winkel=homeWinkel;
        symbol.PositionSetzen(posX, posY, winkel);
    }

    /**
     * Bewegt die Turtle nach vorne.
     * @param laenge Anzahl der Längeneinheiten
     */
    public void Gehen(double laenge)
    {

        int neuX =(int) (Math.round(posX + Math.cos(BogenmassGeben(winkel))*laenge));
        int neuY =(int) (Math.round(posY - Math.sin(BogenmassGeben(winkel))*laenge));
        symbol.PositionSetzen(neuX, neuY, winkel);

        if (stiftUnten) oberflaeche.LinieZeichnen( posX, posY,  neuX,  neuY, farbe);
        posX = neuX;
        posY = neuY;
        if(!schnell)
        {
            long ende = System.currentTimeMillis() + wartezeit;
            while (System.currentTimeMillis() < ende)
            {
                try
                {
                    Thread.sleep(ende - System.currentTimeMillis());
                }
                catch (Exception e){};
            }
        }
    }

    /**
     * Dreht die Turtle
     * @param grad Drehwinkel im Gradmass
     */
    public void Drehen(int grad)
    {
        winkel = winkel + grad;
        symbol.PositionSetzen(posX, posY, winkel);

       if(!schnell)
        {
            long ende = System.currentTimeMillis() + wartezeit;
            while (System.currentTimeMillis() < ende)
            {
                try
                {
                    Thread.sleep(ende - System.currentTimeMillis());
                }
                catch (Exception e){};
            }
        }
    }

    /**
     * Versetzt Zeichenfläche und Turtle in den Ausgangszustand
     */
    public void Loeschen()
    {
        oberflaeche.Loeschen();
        ZumStartpunktGehen();
    }

    /**
     * Setzt die Farbe des Turtle-Stifts (RGB-Code)
     * @param r Rotanteil (0..255)
     * @param g Grünanteil (0..255)
     * @param b Blauanteil (0..255)
     */
    public void StiftfarbeSetzen(int r, int g, int b)
    {
        farbe = new Color (r,g,b);
        symbol.StiftfarbeSetzen(farbe);
    }

    /**
     * Turtle wechselt in den Modus "nicht zeichnen"
     */
    public void StiftHeben()
    {
        stiftUnten=false;
    }

    /**
     * Turtle wechselt in den Modus "zeichnen"
     */
    public void StiftSenken()
    {
        stiftUnten=true;
    }

    /**
     * Gibt den aktuellen Winkel der Turtle zurück
     * @return Winkel im Gradmass
     */
    public int WinkelGeben()
    {
        return winkel;
    }

    /**
     * Gibt die x-Koordinate der Turtle zurück
     * @return x-Koordinate
     */
    public int XPositionGeben()
    {
        return posX;
    }

    /**
     * Gibt die y-Koordinate der Turtle zurück
     * @return y-Koordinate
     */ 
    public int YPositionGeben()
    {
        return posY;
    }

    private double BogenmassGeben(int winkel)
    {
        return winkel*Math.PI/180;
    }

    /**
     * Turtle ohne Zeitverzögerung
     */
    public void SchnellSetzen()
    {
        schnell = true;
    }

    /**
     * Turtle mit Zeitverzögerung
     */
    public void LangsamSetzen()
    {
        schnell = false;
    }

    /**
     * Taktdauer setzen für den Modus mit Zeitverzögerung
     * @param ms Taktdauer
     */
    public void WartezeitSetzen(int ms)
    {
        wartezeit = ms;
    }

    public class Turtlesymbol extends JPanel
    {
        int winkel;
        int x,y;
        int groesse;
        Color c;

        public Turtlesymbol(Oberflaeche o, int xNeu, int yNeu, int winkelNeu)
        {
            winkel =winkelNeu;
            groesse = 30;
            x=xNeu;
            y=yNeu;

            setSize(30,30);
            setMaximumSize(new Dimension(30,30));
            setMinimumSize(new Dimension(30,30));

            setBackground(new Color(0,0,0,0));
            o.add(this);
            setLocation(xNeu,yNeu);
            setVisible(true);

        }

        void PositionSetzen(int xneu, int yneu, int winkelneu)
        {
            setVisible(false);
            x = xneu;
            y = yneu;
            winkel = winkelneu;
            setLocation((int)(xneu-RadiusGeben()*(1-Math.cos(bogen(winkel)))),(int)(yneu-RadiusGeben()*Math.sin(bogen(winkel))-15));
            setVisible(true);
            repaint();

        }

        public void paint(Graphics g)
        {
            int groesse;
            int radius;
            groesse = 40;
            if(getWidth() < groesse)
            {
                groesse=getWidth();
            }

            radius = groesse/2;
            g.setColor(Color.red);
            Graphics2D g2 = (Graphics2D) g;

            g2.rotate(-winkel * Math.PI / 180.0,15,15);
            //Kopf
            g.setColor(new Color(0,200,0));
            g2.fillOval(20,10,10,10);

            //Beine
            g2.fillOval(9,2,5,7);
            g2.fillOval(16,2,5,7);
            g2.fillOval(9,19,5,7);
            g2.fillOval(16,19,5,7);

            g.setColor(c);
            //Augen
            g2.fillRect(27,11,2,3);
            g2.fillRect(27,16,2,3);

            //Schwanz
            g2.fillOval(0,12,10,4);
            //Rumpf
            g.setColor(new Color(139,69,19));

            g2.fillOval(3,5,23,18);

        }

        int RadiusGeben()
        {
            return (int)(groesse/2);
        }

        void StiftfarbeSetzen(Color cNeu)
        {
            c=cNeu;
            repaint();
        }

        private double bogen(double winkel)
        {
            return winkel*Math.PI/180;
        }
    }

    public class Oberflaeche extends JFrame implements WindowListener{

        private Image image;
        private Oberflaeche o;
        private Leinwand leinwand;

        private Oberflaeche(String name, int width, int height) {
            super(name);
            setPreferredSize(new Dimension(width, height));
            pack();

            addWindowListener(this);
            setVisible(true);
            leinwand = new Leinwand(this);
        }

        private Oberflaeche() {
            super("Turtle");
            setPreferredSize(new Dimension(400,400));
            pack();

            addWindowListener(this);
            setVisible(true);
            leinwand = new Leinwand(this);
        }

        public Oberflaeche OberflaecheGeben()
        {
            return new Oberflaeche();
        }

        public void Warten() {
            while (isVisible()) {
                try {
                    Thread.sleep(100);
                } catch (InterruptedException e) {
                    setVisible(false);
                }
            }
        }

        public void windowActivated(WindowEvent e) {    
            paint(getGraphics());
        }

        public void windowClosed(WindowEvent e) {
        }

        public void windowClosing(WindowEvent e) {
            setVisible(false);
            dispose();
        }

        public void windowDeactivated(WindowEvent e) {
        }

        public void windowDeiconified(WindowEvent e) {
            paint(getGraphics());
        }

        public void windowIconified(WindowEvent e) {
        }

        public void windowOpened(WindowEvent e) {
        }

        public void paint(Graphics graphics) {
            super.paint(graphics);
            graphics.drawImage(image, 0, 0, Color.white, null);
        }

        public void LeinwandAktivieren()
        {
            leinwand.setVisible(false);
            leinwand = new Leinwand(this);
            leinwand.setVisible(true);

            add(leinwand);
            pack();
        }

        public void LinieZeichnen(int x1, int y1, int x2, int y2, Color farbe)
        {
            leinwand.LinieZeichnen(x1,y1,x2,y2, farbe);
        }

        public void Loeschen()
        {
            leinwand.Loeschen();
        }

        public class Leinwand extends JPanel

        {

            double hoehe, breite;
            Graphics2D g;
            java.awt.image.BufferedImage img ;

            /**
             * Constructor for objects of class TURTLESYMBOL
             */
            public Leinwand(Oberflaeche cont)
            {
                super();
                hoehe = 400;
                breite = 400;

                setSize((int)hoehe, (int)breite);
                setBackground(Color.red);

                int width = 2000, height = 2000;
                img = GraphicsEnvironment.getLocalGraphicsEnvironment().getDefaultScreenDevice().getDefaultConfiguration().createCompatibleImage( width, height );
                g = img.createGraphics();
                g.setColor( Color.WHITE );
                g.fillRect(0,0,width, height);

            }

            void LinieZeichnen(int x1neu, int y1neu, int x2neu, int y2neu, Color c)
            {

                g.setColor(c);
                g.drawLine(x1neu, y1neu, x2neu, y2neu);
                repaint();
            }

            public void paint(Graphics g) {

                g.drawImage(img,0,0,this);
            }

            public void Loeschen()
            {
                g.setColor(Color.white);
                g.fillRect(0,0,2000,2000);
                repaint();
            }

        }
    }
}
